@@ -71,6 +71,10 @@ int MMG3D_Set_inputSolName(MMG5_pMesh mesh,MMG5_pSol sol, const char* solin) {
   return MMG5_Set_inputSolName(mesh,sol,solin);
 }
 
+int MMG3D_Set_inputParamName(MMG5_pMesh mesh, const char* fparamin) {
+  return MMG5_Set_inputParamName(mesh,fparamin);
+}
+
 int MMG3D_Set_outputMeshName(MMG5_pMesh mesh, const char* meshout) {
 
   return MMG5_Set_outputMeshName(mesh,meshout);
@@ -200,7 +204,7 @@ int MMG3D_Set_solsAtVerticesSize(MMG5_pMesh mesh, MMG5_pSol *sol,int nsols,
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
+ * \param mesh pointer to the mesh structure.
  * \param np number of vertices.
  * \param ne number of tetrahedra.
  * \param nprism number of prisms.
@@ -941,8 +945,8 @@ int  MMG3D_Set_triangles(MMG5_pMesh mesh, MMG5_int *tria, MMG5_int *refs) {
     j = (i-1)*3;
     ptt = &mesh->tria[i];
     ptt->v[0] = tria[j]  ;
-    ptt->v[1] = tria[j+2];
-    ptt->v[2] = tria[j+1];
+    ptt->v[1] = tria[j+1];
+    ptt->v[2] = tria[j+2];
     if ( refs != NULL )
       ptt->ref  = refs[i-1];
   }
@@ -2231,12 +2235,7 @@ int MMG3D_Set_iparameter(MMG5_pMesh mesh, MMG5_pSol sol, int iparam,MMG5_int val
     MMG5_SAFE_CALLOC(mesh->info.par,mesh->info.npar,MMG5_Par,return 0);
 
     MMG5_int inival;
-    if ( sizeof(MMG5_int) == 8 ) {
-      inival = LONG_MAX;
-    }
-    else {
-      inival = INT_MAX;
-    }
+    inival = MMG5_INTMAX;
 
     for (k=0; k<mesh->info.npar; k++) {
       mesh->info.par[k].elt   = MMG5_Noentity;
@@ -2418,6 +2417,13 @@ int MMG3D_Set_dparameter(MMG5_pMesh mesh, MMG5_pSol sol, int dparam, double val)
   case MMG3D_DPARAM_ls :
     mesh->info.ls       = val;
     break;
+  case MMG3D_DPARAM_xreg :
+    if (val < 0.0 || val > 1.0) {
+      fprintf(stderr,"\n  ## Error: %s: Coordinate regularization parameter must be comprised between 0 and 1.\n",__func__);
+    }
+    else
+      mesh->info.lxreg    = val;
+    break;
   case MMG3D_DPARAM_rmc :
     if ( !val ) {
       /* Default value */
@@ -2524,8 +2530,8 @@ int MMG3D_Set_localParameter(MMG5_pMesh mesh,MMG5_pSol sol, int typ, MMG5_int re
   return 1;
 }
 
-int MMG3D_Set_multiMat(MMG5_pMesh mesh,MMG5_pSol sol,MMG5_int ref,int split,MMG5_int rin,MMG5_int rout) {
-  return MMG5_Set_multiMat(mesh,sol,ref,split,rin,rout);
+int MMG3D_Set_multiMat(MMG5_pMesh mesh,MMG5_pSol sol,MMG5_int ref,int split,MMG5_int rmin,MMG5_int rplus) {
+  return MMG5_Set_multiMat(mesh,sol,ref,split,rmin,rplus);
 }
 
 int MMG3D_Set_lsBaseReference(MMG5_pMesh mesh,MMG5_pSol sol,MMG5_int br){
