@@ -161,6 +161,10 @@ int MMG5_chkcol_int(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t iface,
         /* Rough evaluation of edge length (doesn't take into account if some of
          * the modified edges of pt0 are boundaries): for a more precise
          * computation, we need to update the edge tags of pt0.  */
+
+        // Algiane 06/24: to check and fix: If the edge is marked as MG_BDY
+        // (keeping in mind that this tag is not always consistent), the curve
+        // length is computed, I don't know if it is wanted.
         ll = MMG5_lenedgspl(mesh,met,jj,pt0);
         if ( (!ll) || (ll > lon) )//LOPTL too small, we need to put greater than 1.41
           return 0;
@@ -174,14 +178,14 @@ int MMG5_chkcol_int(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t iface,
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param start tetra from which we start to travel
  * \param end tetra at which we stop the travel
  * \param na edge vertex
  * \param nb edge vertex
  * \param piv global index of the pivot to set the sense of travel
- * \param iel pointer toward the last element of the shell
- * \param iopp pointer toward the ending boundary face of the shell
+ * \param iel pointer to the last element of the shell
+ * \param iopp pointer to the ending boundary face of the shell
  *
  * \return -1 if fail, \a piv otherwise.
  *
@@ -230,7 +234,7 @@ MMG5_int MMG3D_unfold_shell(MMG5_pMesh  mesh,MMG5_int start,MMG5_int end, MMG5_i
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
+ * \param mesh pointer to the mesh structure.
  * \param k index of the starting tetra.
  * \param iface local index of the starting face in the tetra \a k.
  * \param ideg local index of the starting edge in the face \a iface.
@@ -354,7 +358,7 @@ MMG5_topchkcol_bdy(MMG5_pMesh mesh,MMG5_int k,int iface,int8_t iedg,MMG5_int *li
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param start tetra from which we start to travel
  * \param na edge vertex
  * \param nb edge vertex
@@ -375,7 +379,7 @@ MMG5_topchkcol_bdy(MMG5_pMesh mesh,MMG5_int k,int iface,int8_t iedg,MMG5_int *li
  */
 static inline
 int MMG3D_get_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_int na, MMG5_int nb,
-                                  int16_t *tag,MMG5_int *ref, MMG5_int piv,MMG5_int adj,
+                                  uint16_t *tag,MMG5_int *ref, MMG5_int piv,MMG5_int adj,
                                   int8_t *filled) {
   MMG5_pTetra  pt;
   MMG5_pxTetra pxt;
@@ -397,6 +401,10 @@ int MMG3D_get_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_int na,
         *tag |= pxt->tag[i];
         *filled = 1;
         return adj;
+      } else if ((pxt->ftag[MMG5_ifar[i][0]] & MG_BDY) || (pxt->ftag[MMG5_ifar[i][1]] & MG_BDY)) {
+        *tag |= (pxt->tag[i] | MG_BDY);
+        *filled = 1;
+        return adj;
       }
     }
 
@@ -416,7 +424,7 @@ int MMG3D_get_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_int na,
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param start tetra from which we start to travel
  * \param ia local index of edge that must be updated
  * \param tag new edge tag
@@ -428,8 +436,7 @@ int MMG3D_get_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_int na,
  * consistent through the edge shell);
  *
  */
-static inline
-int MMG3D_get_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,int16_t *tag,MMG5_int *ref) {
+int MMG3D_get_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,uint16_t *tag,MMG5_int *ref) {
   MMG5_pTetra  pt;
   MMG5_pxTetra pxt;
   MMG5_int     piv,na,nb,adj,*adja;
@@ -445,10 +452,15 @@ int MMG3D_get_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,int16_t *t
 
   if ( pt->xt ) {
     pxt = &mesh->xtetra[pt->xt];
+    *ref = pxt->edg[ia];
     if ( pxt->tag[ia] & MG_BDY ) {
-      *tag |= pxt->tag[ia];
-      *ref = pxt->edg[ia];
-      return 1;
+        *tag |= pxt->tag[ia];
+        *ref = pxt->edg[ia];
+        return 1;
+    } else if ((pxt->ftag[MMG5_ifar[ia][0]] & MG_BDY) || (pxt->ftag[MMG5_ifar[ia][1]] & MG_BDY)) {
+        *tag |= (pxt->tag[ia] | MG_BDY);
+        *ref = pxt->edg[ia];
+        return 1;
     }
   }
 
@@ -482,14 +494,14 @@ int MMG3D_get_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,int16_t *t
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
- * \param met pointer toward the metric structure.
+ * \param mesh pointer to the mesh structure.
+ * \param met pointer to the metric structure.
  * \param k index of element in which we collapse.
  * \param iface face through wich we perform the collapse
  * \param iedg edge to collapse (in local face num)
- * \param listv pointer toward the list of the tetra in the ball of \a p0.
+ * \param listv pointer to the list of the tetra in the ball of \a p0.
  * \param ilistv number of tetra in the ball of \a p0.
- * \param lists pointer toward the surfacic ball of \a p0.
+ * \param lists pointer to the surfacic ball of \a p0.
  * \param ilists number of tetra in the surfacic ball of \a p0.
  * \param refmin reference of one of the two subdomains in presence
  * \param refplus reference of the other subdomain in presence
@@ -523,7 +535,7 @@ int MMG5_chkcol_bdy(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t iface,
   double       ps,devold,devnew,hmax,hausd;
   MMG5_int     nump,numq,ndepmin,ndepplus,l,kk,iel;
   int          nr,nbbdy,isloc,iedgeOpp,ipp;
-  int16_t      tag;
+  uint16_t     tag;
   int8_t       iopp,iopp2,ia,ip,i,iq,i0,i1,ier,isminp,isplp;
 #ifndef NDEBUG
   MMG5_pPoint  p0;
@@ -613,38 +625,36 @@ int MMG5_chkcol_bdy(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t iface,
       if ( pt->xt && (pxt->ftag[ipp] & MG_BDY) && (pxt->ftag[iq] & MG_BDY) )
         return 0;
 
-      if ( pt->xt )  {
-        for (i=0; i<4; i++) {
-          if ( i==ipp || i==iq ) {
-            continue;
+      for (i=0; i<4; i++) {
+        if ( i==ipp || i==iq ) {
+          continue;
+        }
+
+        /*  Avoid surface crimping: check that the collapse doesn't merge 3
+         *  bdy edge along a non bdy face: we have to check the edge of each
+         *  shell because some MG_BDY tags may be missings due to the creation
+         *  of an xtetra during a previous collapse */
+        if ( (!pt->xt) || !(pxt->ftag[i] & MG_BDY) ) {
+          uint16_t  tag0,tag1,tag2;
+          MMG5_int ref0,ref1,ref2;
+
+          tag0 = tag1 = tag2 = 0;
+          ref0 = ref1 = ref2 = 0;
+
+          if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][0],&tag0,&ref0) ) {
+            fprintf(stderr,"\n  ## Error: %s: 0. unable to get edge info.\n",__func__);
+            return 0;
           }
-
-          /*  Avoid surface crimping: check that the collapse doesn't merge 3
-           *  bdy edge along a non bdy face: we have to check the edge of each
-           *  shell because some MG_BDY tags may be missings due to the creation
-           *  of an xtetra during a previous collapse */
-          if ( !(pxt->ftag[i] & MG_BDY) ) {
-            int16_t  tag0,tag1,tag2;
-            MMG5_int ref0,ref1,ref2;
-
-            tag0 = tag1 = tag2 = 0;
-            ref0 = ref1 = ref2 = 0;
-
-            if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][0],&tag0,&ref0) ) {
-              fprintf(stderr,"\n  ## Error: %s: 0. unable to get edge info.\n",__func__);
-              return 0;
-            }
-            if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][1],&tag1,&ref1) ) {
-              fprintf(stderr,"\n  ## Error: %s: 1. unable to get edge info.\n",__func__);
-              return 0;
-            }
-            if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][2],&tag2,&ref2) ) {
-              fprintf(stderr,"\n  ## Error: %s: 2. unable to get edge info.\n",__func__);
-              return 0;
-            }
-            if ( (tag0 & MG_BDY) && (tag1 & MG_BDY) && (tag2 & MG_BDY ) ) {
-              return 0;
-            }
+          if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][1],&tag1,&ref1) ) {
+            fprintf(stderr,"\n  ## Error: %s: 1. unable to get edge info.\n",__func__);
+            return 0;
+          }
+          if ( !MMG3D_get_shellEdgeTag(mesh,iel,MMG5_iarf[i][2],&tag2,&ref2) ) {
+            fprintf(stderr,"\n  ## Error: %s: 2. unable to get edge info.\n",__func__);
+            return 0;
+          }
+          if ( (tag0 & MG_BDY) && (tag1 & MG_BDY) && (tag2 & MG_BDY ) ) {
+            return 0;
           }
         }
       }
@@ -944,7 +954,7 @@ void MMG3D_update_edgeTag(MMG5_pTetra pt,MMG5_pxTetra pxt,MMG5_int np, MMG5_int
   int      i,j;
   MMG5_int p0,p1;
   uint8_t  ia,iav;
-  int16_t  tag,tag1;
+  uint16_t tag,tag1;
 
   /* update tags for edges */
   for ( j=0; j<3; j++ ) {
@@ -983,7 +993,7 @@ void MMG3D_update_edgeTag(MMG5_pTetra pt,MMG5_pxTetra pxt,MMG5_int np, MMG5_int
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param start tetra from which we start to travel
  * \param na edge vertex
  * \param nb edge vertex
@@ -1000,11 +1010,11 @@ void MMG3D_update_edgeTag(MMG5_pTetra pt,MMG5_pxTetra pxt,MMG5_int np, MMG5_int
  */
 static inline
 MMG5_int MMG3D_update_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_int na, MMG5_int nb,
-                                     int16_t tag,MMG5_int ref, MMG5_int piv,MMG5_int adj) {
+                                     uint16_t tag,MMG5_int ref, MMG5_int piv,MMG5_int adj) {
   MMG5_pTetra  pt;
   MMG5_pxTetra pxt;
   MMG5_int     *adja;
-  int16_t      xtag;
+  uint16_t     xtag;
   int8_t       i;
 
   assert ( tag & MG_BDY && "Unexpected non boundary tag");
@@ -1053,7 +1063,7 @@ MMG5_int MMG3D_update_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param start tetra from which we start to travel
  * \param ia local index of edge that must be updated
  * \param tag new edge tag
@@ -1064,11 +1074,11 @@ MMG5_int MMG3D_update_shellEdgeTag_oneDir(MMG5_pMesh  mesh,MMG5_int start, MMG5_
  *
  */
 static inline
-int MMG3D_update_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,int16_t tag,MMG5_int ref) {
+int MMG3D_update_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,uint16_t tag,MMG5_int ref) {
   MMG5_pTetra  pt;
   MMG5_pxTetra pxt;
   MMG5_int     piv,na,nb,adj,*adja;
-  int16_t      xtag;
+  uint16_t     xtag;
 
   pt   = &mesh->tetra[start];
 
@@ -1126,9 +1136,9 @@ int MMG3D_update_shellEdgeTag(MMG5_pMesh  mesh,MMG5_int start, int8_t ia,int16_t
 }
 
 /**
- * \param mesh pointer toward the mesh
- * \param met pointer toward the metric
- * \param list pointer toward the ball of the point
+ * \param mesh pointer to the mesh
+ * \param met pointer to the metric
+ * \param list pointer to the ball of the point
  * \param ilist number of elements in the ball of the point
  * \param indq local index of the point on which we collapse
  * \param typchk type of check performed depending on the remeshing step
@@ -1268,9 +1278,9 @@ MMG5_int MMG5_colver(MMG5_pMesh mesh,MMG5_pSol met,int64_t *list,int ilist,int8_
       int iped = coled[l+0];
       int iqed = coled[l+1];
 
-      int16_t  tagip = 0;
+      uint16_t tagip = 0;
       MMG5_int refip = 0;
-      int16_t  tagiq = 0;
+      uint16_t tagiq = 0;
       MMG5_int refiq = 0;
 
       if ( !MMG3D_get_shellEdgeTag(mesh,iel,iped,&tagip,&refip) ) {
