@@ -39,14 +39,14 @@
 /**
  * \param imprim verbosity level (muted for stdout if -1)
  * \param filename file to open
- * \param inm pointer toward the file unit
+ * \param inm pointer to the file unit
  * \param bin 1 if file will be at binary format
  * \param modeASCII mode in which to open an ascii file ("r","r+","w","w+",...)
  * \param modeASCII mode in which to open an ascii file ("r","r+","w","w+",...)
  *
- * \return 0 if fail to open file, -1 for other errors, 1 if success.
+ * \return 0 if the file could not be opened, -1 for other errors, 1 if success.
  *
- * Try to open a Medit file at asked mode (read only, write, etc) and store if
+ * Try to open a Medit file in the requested mode (read only, write, etc) and store if
  * file is binary (depending on the extension).
  *
  */
@@ -136,6 +136,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
   ncor = ned = npreq = ntreq = nqreq = nereq = nedreq = nr = ng = 0;
   nppar = nedpar = ntpar = nqpar = nepar = 0;
   iswp = 0;
+  bpos = ia = idn = ip = 0;
   mesh->np = mesh->nt = mesh->ne = 0;
   nref = 0;
 
@@ -144,7 +145,11 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
     strcpy(chaine,"D");
     while(fscanf(inm,"%127s",&chaine[0])!=EOF && strncmp(chaine,"End",strlen("End")) ) {
       if ( chaine[0] == '#' ) {
-        fgets(strskip,MMG5_FILESTR_LGTH,inm);
+        while(1){           // skip until end of line or file
+          char *s = fgets(strskip,MMG5_FILESTR_LGTH,inm);
+          if(!s) break;     // nothing could be read
+          if(s[strlen(s)-1]=='\n') break;   // end of line
+        }
         continue;
       }
 
@@ -274,7 +279,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->npi,MMG5_SW,1,inm);
-        if(iswp) mesh->npi=MMG5_SWAPBIN(mesh->npi);
+        if(iswp) mesh->npi=MMG5_swapbin(mesh->npi);
         posnp = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -283,7 +288,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&npreq,MMG5_SW,1,inm);
-        if(iswp) npreq=MMG5_SWAPBIN(npreq);
+        if(iswp) npreq=MMG5_swapbin(npreq);
         posnpreq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -292,7 +297,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nti,MMG5_SW,1,inm);
-        if(iswp) mesh->nti=MMG5_SWAPBIN(mesh->nti);
+        if(iswp) mesh->nti=MMG5_swapbin(mesh->nti);
         posnt = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -301,7 +306,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&ntreq,MMG5_SW,1,inm);
-        if(iswp) ntreq=MMG5_SWAPBIN(ntreq);
+        if(iswp) ntreq=MMG5_swapbin(ntreq);
         posntreq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -311,7 +316,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nquad,MMG5_SW,1,inm);
-        if(iswp) mesh->nquad=MMG5_SWAPBIN(mesh->nquad);
+        if(iswp) mesh->nquad=MMG5_swapbin(mesh->nquad);
         posnq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -320,7 +325,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&nqreq,MMG5_SW,1,inm);
-        if(iswp) nqreq=MMG5_SWAPBIN(nqreq);
+        if(iswp) nqreq=MMG5_swapbin(nqreq);
         posnqreq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -329,7 +334,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nei,MMG5_SW,1,inm);
-        if(iswp) mesh->nei=MMG5_SWAPBIN(mesh->nei);
+        if(iswp) mesh->nei=MMG5_swapbin(mesh->nei);
         posne = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -338,7 +343,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nprism,MMG5_SW,1,inm);
-        if(iswp) mesh->nprism=MMG5_SWAPBIN(mesh->nprism);
+        if(iswp) mesh->nprism=MMG5_swapbin(mesh->nprism);
         posnprism = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -347,7 +352,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&nereq,MMG5_SW,1,inm);
-        if(iswp) nereq=MMG5_SWAPBIN(nereq);
+        if(iswp) nereq=MMG5_swapbin(nereq);
         posnereq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -356,7 +361,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&ncor,MMG5_SW,1,inm);
-        if(iswp) ncor=MMG5_SWAPBIN(ncor);
+        if(iswp) ncor=MMG5_swapbin(ncor);
         posncor = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -365,7 +370,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nai,MMG5_SW,1,inm);
-        if(iswp) mesh->nai=MMG5_SWAPBIN(mesh->nai);
+        if(iswp) mesh->nai=MMG5_swapbin(mesh->nai);
         posned = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -374,7 +379,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&nedreq,MMG5_SW,1,inm);
-        if(iswp) nedreq=MMG5_SWAPBIN(nedreq);
+        if(iswp) nedreq=MMG5_swapbin(nedreq);
         posnedreq = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -383,7 +388,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&nr,MMG5_SW,1,inm);
-        if(iswp) nr=MMG5_SWAPBIN(nr);
+        if(iswp) nr=MMG5_swapbin(nr);
         posnr = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -392,7 +397,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&ng,MMG5_SW,1,inm);
-        if(iswp) ng=MMG5_SWAPBIN(ng);
+        if(iswp) ng=MMG5_swapbin(ng);
         posnormal = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -401,7 +406,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         MMG_FREAD(&bpos,MMG5_SW,1,inm); //NulPos
         if(iswp) bpos=MMG5_swapbin(bpos);
         MMG_FREAD(&mesh->nc1,MMG5_SW,1,inm);
-        if(iswp) mesh->nc1=MMG5_SWAPBIN(mesh->nc1);
+        if(iswp) mesh->nc1=MMG5_swapbin(mesh->nc1);
         posnc1 = ftell(inm);
         rewind(inm);
         fseek(inm,bpos,SEEK_SET);
@@ -450,7 +455,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
           ppt->c[i] = (double) fc;
         }
         MMG_FREAD(&ppt->ref,MMG5_SW,1,inm);
-        if(iswp) ppt->ref=MMG5_SWAPBIN(ppt->ref);
+        if(iswp) ppt->ref=MMG5_swapbin(ppt->ref);
       }
     } else {
       if (!bin) {
@@ -462,7 +467,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
           if(iswp) ppt->c[i]=MMG5_swapd(ppt->c[i]);
         }
         MMG_FREAD(&ppt->ref,MMG5_SW,1,inm);
-        if(iswp) ppt->ref=MMG5_SWAPBIN(ppt->ref);
+        if(iswp) ppt->ref=MMG5_swapbin(ppt->ref);
       }
     }
 
@@ -554,10 +559,10 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
       else {
         for (i=0 ; i<3 ; i++) {
           MMG_FREAD(&pt1->v[i],MMG5_SW,1,inm);
-          if(iswp) pt1->v[i]=MMG5_SWAPBIN(pt1->v[i]);
+          if(iswp) pt1->v[i]=MMG5_swapbin(pt1->v[i]);
         }
         MMG_FREAD(&pt1->ref,MMG5_SW,1,inm);
-        if(iswp) pt1->ref=MMG5_SWAPBIN(pt1->ref);
+        if(iswp) pt1->ref=MMG5_swapbin(pt1->ref);
       }
       if ( pt1->ref < 0 ) {
         pt1->ref = -pt1->ref;
@@ -626,10 +631,10 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
       else {
         for (i=0 ; i<4 ; i++) {
           MMG_FREAD(&pq1->v[i],MMG5_SW,1,inm);
-          if(iswp) pq1->v[i]=MMG5_SWAPBIN(pq1->v[i]);
+          if(iswp) pq1->v[i]=MMG5_swapbin(pq1->v[i]);
         }
         MMG_FREAD(&pq1->ref,MMG5_SW,1,inm);
-        if(iswp) pq1->ref=MMG5_SWAPBIN(pq1->ref);
+        if(iswp) pq1->ref=MMG5_swapbin(pq1->ref);
       }
       if ( pq1->ref < 0 ) {
         pq1->ref = -pq1->ref;
@@ -702,11 +707,11 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
       }
       else {
         MMG_FREAD(&pa->a,MMG5_SW,1,inm);
-        if(iswp) pa->a=MMG5_SWAPBIN(pa->a);
+        if(iswp) pa->a=MMG5_swapbin(pa->a);
         MMG_FREAD(&pa->b,MMG5_SW,1,inm);
-        if(iswp) pa->b=MMG5_SWAPBIN(pa->b);
+        if(iswp) pa->b=MMG5_swapbin(pa->b);
         MMG_FREAD(&pa->ref,MMG5_SW,1,inm);
-        if(iswp) pa->ref=MMG5_SWAPBIN(pa->ref);
+        if(iswp) pa->ref=MMG5_swapbin(pa->ref);
       }
       pa->tag |= MG_REF;
       if ( pa->ref < 0 ) {
@@ -725,7 +730,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         }
         else {
           MMG_FREAD(&ia,MMG5_SW,1,inm);
-          if(iswp) ia=MMG5_SWAPBIN(ia);
+          if(iswp) ia=MMG5_swapbin(ia);
         }
         if(ia>na) {
           fprintf(stderr,"\n  ## Warning: %s: ridge number %8" MMG5_PRId " ignored.\n",
@@ -746,7 +751,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         }
         else {
           MMG_FREAD(&ia,MMG5_SW,1,inm);
-          if(iswp) ia=MMG5_SWAPBIN(ia);
+          if(iswp) ia=MMG5_swapbin(ia);
         }
         if(ia>na) {
           fprintf(stderr,"\n  ## Warning: %s: required Edges number %8" MMG5_PRId "/%8" MMG5_PRId ""
@@ -767,7 +772,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         }
         else {
           MMG_FREAD(&ia,MMG5_SW,1,inm);
-          if(iswp) ia=MMG5_SWAPBIN(ia);
+          if(iswp) ia=MMG5_swapbin(ia);
         }
         if(ia>na) {
           fprintf(stderr,"\n  ## Warning: %s: parallel Edges number %8" MMG5_PRId "/%8" MMG5_PRId ""
@@ -792,10 +797,10 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
     else {
       for (i=0 ; i<4 ; i++) {
         MMG_FREAD(&pt->v[i],MMG5_SW,1,inm);
-        if(iswp) pt->v[i]=MMG5_SWAPBIN(pt->v[i]);
+        if(iswp) pt->v[i]=MMG5_swapbin(pt->v[i]);
       }
       MMG_FREAD(&ref,MMG5_SW,1,inm);
-      if(iswp) ref=MMG5_SWAPBIN(ref);
+      if(iswp) ref=MMG5_swapbin(ref);
     }
     if(ref < 0) {
       nref++;
@@ -806,7 +811,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
       ppt->tag &= ~MG_NUL;
     }
 
-    /* Possibly switch 2 vertices number so that each tet is positively oriented */
+    /* Possibly switch 2 vertex numbers so that each tet is positively oriented */
     if ( MMG5_orvol(mesh->point,pt->v) < 0.0 ) {
       /* mesh->xt temporary used to count reoriented tetra */
       mesh->xt++;
@@ -875,10 +880,10 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
     else {
       for (i=0 ; i<6 ; i++) {
         MMG_FREAD(&pp->v[i],MMG5_SW,1,inm);
-        if(iswp) pp->v[i]=MMG5_SWAPBIN(pp->v[i]);
+        if(iswp) pp->v[i]=MMG5_swapbin(pp->v[i]);
       }
       MMG_FREAD(&ref,MMG5_SW,1,inm);
-      if(iswp) ref=MMG5_SWAPBIN(ref);
+      if(iswp) ref=MMG5_swapbin(ref);
     }
     pp->ref  = ref;
     if ( pp-> ref < 0 ) {
@@ -956,9 +961,9 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
         }
         else {
           MMG_FREAD(&ip,MMG5_SW,1,inm);
-          if(iswp) ip=MMG5_SWAPBIN(ip);
+          if(iswp) ip=MMG5_swapbin(ip);
           MMG_FREAD(&idn,MMG5_SW,1,inm);
-          if(iswp) idn=MMG5_SWAPBIN(idn);
+          if(iswp) idn=MMG5_swapbin(idn);
         }
         if ( idn > 0 && ip < mesh->np+1 ) {
           if ( (mesh->info.iso ) &&  mesh->point[ip].xp == -1 ) {
@@ -1032,7 +1037,7 @@ int MMG3D_loadMesh_opened(MMG5_pMesh mesh,FILE *inm,int bin) {
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
+ * \param mesh pointer to the mesh structure.
  * \param filename name of file.
  *
  * \return 0 if the file is not found, -1 if we detect mismatch parameters or we
@@ -1115,6 +1120,15 @@ int MMG3D_loadMshMesh(MMG5_pMesh mesh,MMG5_pSol sol,const char *filename) {
   return ier;
 }
 
+/* This is an API function. It is documented in the header.
+ *
+ * This function reads a 3D mesh and 0 or 1 data fields in MSH file format (.msh
+ * extension). We read only low-order points, edges, triangles, quadrangles,
+ * tetrahedra and prisms.
+ *
+ * It returns 0 if the file is not found, -1 if it fails for another reason (mem
+ * lack, file format...), 1 if success.
+ */
 int MMG3D_loadMshMesh_and_allData(MMG5_pMesh mesh,MMG5_pSol *sol,const char *filename) {
   FILE*       inm;
   int         ier;
@@ -1173,7 +1187,7 @@ int MMG3D_loadMshMesh_and_allData(MMG5_pMesh mesh,MMG5_pSol *sol,const char *fil
   return ier;
 }
 
-int MMG3D_loadGenericMesh(MMG5_pMesh mesh, MMG5_pSol sol, const char *filename) {
+int MMG3D_loadGenericMesh(MMG5_pMesh mesh, MMG5_pSol met, MMG5_pSol sol, const char *filename) {
   int ier=0;
   const char *filenameptr,*solnameptr;
   char *tmp,*soltmp;
@@ -1212,18 +1226,18 @@ int MMG3D_loadGenericMesh(MMG5_pMesh mesh, MMG5_pSol sol, const char *filename)
     break;
 
   case ( MMG5_FMT_VtkVtu ):
-    ier = MMG3D_loadVtuMesh(mesh,sol,tmp);
+    ier = MMG3D_loadVtuMesh(mesh,met,sol,tmp);
     break;
 
   case ( MMG5_FMT_VtkVtk ):
-    ier = MMG3D_loadVtkMesh(mesh,sol,tmp);
+    ier = MMG3D_loadVtkMesh(mesh,met,sol,tmp);
     break;
 
   case ( MMG5_FMT_MeditASCII ): case ( MMG5_FMT_MeditBinary ):
     ier = MMG3D_loadMesh(mesh,tmp);
     if ( ier <  1 ) { break; }
 
-    /* Facultative metric */
+    /* Optional metric */
     if ( sol ) {
       MMG5_SAFE_MALLOC(soltmp,strlen(solnameptr)+1,char,return -1);
       strcpy(soltmp,solnameptr);
@@ -1247,8 +1261,8 @@ int MMG3D_loadGenericMesh(MMG5_pMesh mesh, MMG5_pSol sol, const char *filename)
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
- * \param filename pointer toward the name of file.
+ * \param mesh pointer to the mesh structure.
+ * \param filename pointer to the name of file.
  * \return 0 if failed, 1 otherwise.
  *
  * Save mesh data.
@@ -1276,7 +1290,7 @@ int MMG3D_saveMesh(MMG5_pMesh mesh, const char *filename) {
     return 0;
   }
 
-  /*entete fichier*/
+  /* file header */
   binch=0; bpos=10;
   if(!bin) {
     strcpy(&chaine[0],"MeshVersionFormatted 2\n");
@@ -2031,7 +2045,7 @@ int MMG3D_saveMesh(MMG5_pMesh mesh, const char *filename) {
     }
   }
 
-  /*fin fichier*/
+  /* end of file */
   if(!bin) {
     strcpy(&chaine[0],"\n\nEnd\n");
     fprintf(inm,"%s",chaine);
@@ -2163,13 +2177,13 @@ int MMG3D_loadSol(MMG5_pMesh mesh,MMG5_pSol met, const char *filename) {
     return ier;
   }
 
-  /* Allocate and store the header informations for each solution */
+  /* Allocate and store the header information for each solution */
   if ( !MMG3D_Set_solSize(mesh,met,MMG5_Vertex,mesh->np,type[0]) ) {
     fclose(inm);
     MMG5_SAFE_FREE(type);
     return -1;
   }
-  /* For binary file, we read the verson inside the file */
+  /* For binary files, we read the verson inside the file */
   if ( ver ) met->ver = ver;
 
   MMG5_SAFE_FREE(type);
@@ -2179,7 +2193,7 @@ int MMG3D_loadSol(MMG5_pMesh mesh,MMG5_pSol met, const char *filename) {
   fseek(inm,posnp,SEEK_SET);
 
   if ( met->ver == 1 ) {
-    /* Simple precision */
+    /* Single precision */
     for (k=1; k<=mesh->np; k++) {
       if ( MMG5_readFloatSol3D(met,inm,bin,iswp,k) < 0 ) return -1;
     }
@@ -2244,7 +2258,7 @@ int MMG3D_loadAllSols(MMG5_pMesh mesh,MMG5_pSol *sol, const char *filename) {
   for ( j=0; j<nsols; ++j ) {
     psl = *sol + j;
 
-    /* Give an arbitrary name to the solution because the Medit format has non
+    /* Give an arbitrary name to the solution because the Medit format has no
      * name field */
     sprintf(data,"sol_%" MMG5_PRId "",j);
     if ( !MMG3D_Set_inputSolName(mesh,psl,data) ) {
@@ -2262,7 +2276,7 @@ int MMG3D_loadAllSols(MMG5_pMesh mesh,MMG5_pSol *sol, const char *filename) {
       return -1;
     }
 
-    /* For binary file, we read the verson inside the file */
+    /* For binary file, we read the version inside the file */
     if ( ver ) psl->ver = ver;
   }
   MMG5_SAFE_FREE(type);
@@ -2272,7 +2286,7 @@ int MMG3D_loadAllSols(MMG5_pMesh mesh,MMG5_pSol *sol, const char *filename) {
   fseek(inm,posnp,SEEK_SET);
 
   if ( (*sol)[0].ver == 1 ) {
-    /* Simple precision */
+    /* Single precision */
     for (k=1; k<=mesh->np; k++) {
       for ( j=0; j<nsols; ++j ) {
         psl = *sol + j;
