@@ -39,10 +39,10 @@
 extern int8_t ddb;
 
 /**
- * \param mesh pointer toward the mesh structure.
- * \param met pointer toward the metric structure.
- * \param list pointer toward the shell of the edge.
- * \param ilist pointer toward the size of the shell of the edge.
+ * \param mesh pointer to the mesh structure.
+ * \param met pointer to the metric structure.
+ * \param list pointer to the shell of the edge.
+ * \param ilist pointer to the size of the shell of the edge.
  * \param it1 first element of the open shell.
  * \param it2 last element of the open shell.
  * \param typchk type of checking permformed for edge length (hmin or LSHORT
@@ -77,12 +77,15 @@ int MMG5_chkswpbdy(MMG5_pMesh mesh, MMG5_pSol met, int64_t *list,int ilist,
   np = pt->v[MMG5_iare[ia][0]];
   nq = pt->v[MMG5_iare[ia][1]];
 
+  // Algiane 05/04/24: I think that the assumption that was previously made that
+  // we can arrive from a tetrahedra without a boundary face (i.e. without an
+  // xtetra) never happens
+  assert ( pt->xt && "Boundary edges have to be swapped from a boundary face" );
+
   /* No swap of geometric edge */
-  if ( pt->xt ) {
-    pxt = &mesh->xtetra[pt->xt];
-    if ( (pxt->edg[ia]>0) || MG_EDG_OR_NOM(pxt->tag[ia]) || (pxt->tag[ia] & MG_REQ) ) {
-      return 0;
-    }
+  pxt = &mesh->xtetra[pt->xt];
+  if ( (pxt->edg[ia]>0) || MG_EDG_OR_NOM(pxt->tag[ia]) || (pxt->tag[ia] & MG_REQ) ) {
+    return 0;
   }
 
   /* No swap when either internal or external component has only 1 element (as
@@ -343,7 +346,26 @@ int MMG5_chkswpbdy(MMG5_pMesh mesh, MMG5_pSol met, int64_t *list,int ilist,
   ppt0->c[1] = 0.5*(p0->c[1] + p1->c[1]);
   ppt0->c[2] = 0.5*(p0->c[2] + p1->c[2]);
 
+#ifndef NDEBUG
+  /* Security check: ensure that the edge is boundary */
+  uint16_t  tag = 0;
+  MMG5_int ref = 0;
+  if ( !MMG3D_get_shellEdgeTag(mesh,list[0]/6,list[0]%6,&tag,&ref) ) {
+    fprintf(stderr,"\n  ## Warning: %s: 0. unable to get edge info"
+            " (tetra %d).\n",__func__,MMG3D_indElt(mesh,list[0]/6));
+    return 0;
+  }
+  assert ( (tag & MG_BDY)  && "Edge should be boundary but is not");
+#endif
+
   if ( met->m ) {
+    pt  = &mesh->tetra[list[0]/6];
+    assert ( pt->xt && "Boundary edge interpolated from non-boundary face");
+
+    /* Mark edge as boundary to ensure suitable detection of bdy edge during
+     * interpolation */
+    mesh->xtetra[pt->xt].tag[list[0]%6] |= MG_BDY;
+
     if ( typchk == 1 && (met->size>1) ) {
       if ( MMG3D_intmet33_ani(mesh,met,list[0]/6,list[0]%6,0,0.5) <= 0 )
         return 0;
@@ -449,13 +471,13 @@ int MMG5_chkswpbdy(MMG5_pMesh mesh, MMG5_pSol met, int64_t *list,int ilist,
 }
 
 /**
- * \param mesh pointer toward the mesh structure
- * \param met pointer toward the solution structure
- * \param list pointer toward the shell of the edge
+ * \param mesh pointer to the mesh structure
+ * \param met pointer to the solution structure
+ * \param list pointer to the shell of the edge
  * \param ret dobble of the number of tetrahedra in the shell
  * \param it1 boundary face carrying the beforehand tested terminal
  * point for collapse
- * \param PROctree pointer toward the PROctree structure in Delaunay mode,
+ * \param PROctree pointer to the PROctree structure in Delaunay mode,
  * NULL pointer in pattern mode.
  * \param typchk type of checking permformed for edge length (hmin or LSHORT
  * criterion).
@@ -583,8 +605,8 @@ int MMG5_swpbdy(MMG5_pMesh mesh,MMG5_pSol met,int64_t *list,int ret,MMG5_int it1
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
- * \param met pointer toward the sol structure.
+ * \param mesh pointer to the mesh structure.
+ * \param met pointer to the sol structure.
  * \param k index of the tetrahedron with multiple boundary faces (to be swapped).
  * \param metRidTyp metric storage (classic or special)
  * \param ifac face of the tetra \a k that give the best results for the swap23
@@ -717,9 +739,20 @@ int MMG3D_swap23(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t metRidTyp,
   }
 
   /** Swap */
+  /* Store useful information from pt1 before overwrite by memcpy*/
   xt1 = pt1->xt;
 
   np    = pt1->v[tau1[0]];
+
+  MMG5_int ref[6] = {0};
+  uint16_t  tag[6] = {0};
+  for (i=0;i<6;i++) {
+    if ( !MMG3D_get_shellEdgeTag(mesh,k1,taued1[i],&tag[i],&ref[i]) ) {
+      fprintf(stderr,"\n  ## Error: %s: %d. unable to get edge info.\n",__func__,i);
+      return 0;
+    }
+  }
+
   memcpy(pt1,pt0,sizeof(MMG5_Tetra));
 
   iel = MMG3D_newElt(mesh);
@@ -857,14 +890,25 @@ int MMG3D_swap23(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t metRidTyp,
     pxt1 = &mesh->xtetra[xt1];
 
     /* Assignation of the xt fields to the appropriate tets */
+    /* Warning: after collapses, some boundary edges not connected to boundary
+     * faces may have a 0 tag inside a xtetra (see \ref MMG5_colver when a
+     * xtetra is assigned to one of the neighbours of the tetra of the edge
+     * shell). In consequence, we cannot simply use the stored tags. */
+
     /* xt[0] */
     xt[0].tag[taued0[0]] = 0;
-    xt[0].tag[taued0[3]] = pxt1->tag[taued1[2]];
-    xt[0].tag[taued0[4]] = pxt1->tag[taued1[1]];
+
+    xt[0].tag[taued0[3]] = tag[2];
+    xt[0].tag[taued0[4]] = tag[1];
+    /* As the edge tag of tetra 0 may be erroneous if the edge doesn't belong to
+     * a boundary face */
+    xt[0].tag[taued0[5]] = tag[5];
+
 
     xt[0].edg[taued0[0]] = 0;
-    xt[0].edg[taued0[3]] =  pxt1->edg[taued1[2]];
-    xt[0].edg[taued0[4]] =  pxt1->edg[taued1[1]];
+    xt[0].edg[taued0[3]] = ref[2];
+    xt[0].edg[taued0[4]] = ref[1];
+    xt[0].edg[taued0[5]] = ref[5];
 
     xt[0].ref[ tau0[0]] = pxt1->ref[tau1[1]];
     xt[0].ref[ tau0[2]] = 0;
@@ -879,12 +923,18 @@ int MMG3D_swap23(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t metRidTyp,
 
     /* xt[1] */
     xt[1].tag[taued0[1]] = 0;
-    xt[1].tag[taued0[3]] = pxt1->tag[taued1[0]];
-    xt[1].tag[taued0[5]] = pxt1->tag[taued1[2]];
+
+    xt[1].tag[taued0[3]] = tag[0];
+    xt[1].tag[taued0[5]] = tag[1];
+
+    /* As the edge tag of tetra 0 may be erroneous if the edge doesn't belong to
+     * a boundary face */
+    xt[1].tag[taued0[4]] = tag[3];
 
     xt[1].edg[taued0[1]] = 0;
-    xt[1].edg[taued0[3]] =  pxt1->edg[taued1[0]];
-    xt[1].edg[taued0[5]] =  pxt1->edg[taued1[2]];
+    xt[1].edg[taued0[3]] = ref[0];
+    xt[1].edg[taued0[5]] = ref[1];
+    xt[1].edg[taued0[4]] = ref[3];
 
     xt[1].ref[ tau0[0]] = pxt1->ref[tau1[3]];
     xt[1].ref[ tau0[1]] = 0;
@@ -899,12 +949,18 @@ int MMG3D_swap23(MMG5_pMesh mesh,MMG5_pSol met,MMG5_int k,int8_t metRidTyp,
 
     /* xt[2] */
     xt[2].tag[taued0[2]] = 0;
-    xt[2].tag[taued0[4]] = pxt1->tag[taued1[0]];
-    xt[2].tag[taued0[5]] = pxt1->tag[taued1[1]];
+
+    xt[2].tag[taued0[4]] = tag[0];
+    xt[2].tag[taued0[5]] = tag[2];
+
+    /* As the edge tag of tetra 0 may be erroneous if the edge doesn't belong to
+     * a boundary face */
+    xt[2].tag[taued0[3]] = tag[4];
 
     xt[2].edg[taued0[2]] = 0;
-    xt[2].edg[taued0[4]] = pxt1->edg[taued1[0]];
-    xt[2].edg[taued0[5]] = pxt1->edg[taued1[1]];
+    xt[2].edg[taued0[4]] = ref[0];
+    xt[2].edg[taued0[5]] = ref[2];
+    xt[2].edg[taued0[3]] = ref[4];
 
     xt[2].ref[ tau0[0]] = pxt1->ref[tau1[2]];
     xt[2].ref[ tau0[1]] = 0;
