@@ -125,8 +125,9 @@ int MMGS_defaultValues(MMG5_pMesh mesh) {
 // In adp mode : -sol or -met or default allow to store the metric.
 int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol sol) {
   MMG5_pSol tmp = NULL;
-  int    i;
-  char   namein[MMG5_FILESTR_LGTH];
+  double val;
+  int    i,param;
+  char   namein[MMG5_FILESTR_LGTH],*endptr;
 
   /* First step: search if user want to see the default parameters values. */
   for ( i=1; i< argc; ++i ) {
@@ -134,6 +135,10 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
       MMGS_defaultValues(mesh);
       return 0;
     }
+    else if ( ( !strcmp( argv[ i ],"-?" ) ) || ( !strcmp( argv[ i ],"-h" ) ) ) {
+      MMGS_usage(argv[0]);
+      return 0;
+    }
   }
 
   /* Second step: read all other arguments. */
@@ -141,57 +146,90 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
   while ( i < argc ) {
     if ( *argv[i] == '-' ) {
       switch(argv[i][1]) {
-      case '?':
-        MMGS_usage(argv[0]);
-        return 0;
-        break;
       case 'a': /* ridge angle */
-        if ( !strcmp(argv[i],"-ar") && ++i < argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_angleDetection,
-                                    atof(argv[i])) )
+        if ( !strcmp(argv[i],"-ar") ) {
+          if ( i >= argc -1 ) {
+            fprintf(stderr,"\nMissing argument option %s\n",argv[i]);
             return 0;
+          }
+          else {
+            val = strtof(argv[i+1],&endptr);
+            if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+              ++i;
+              if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_angleDetection,val))
+                return 0;
+            }
+            else {
+              /* argument is not a number */
+              fprintf(stderr,"\nMissing argument option %s\n",argv[i]);
+              return 0;
+            }
+          }
         }
         break;
       case 'A': /* anisotropy */
         if ( !MMGS_Set_solSize(mesh,met,MMG5_Vertex,0,MMG5_Tensor) )
           return 0;
         break;
+      case 'f':
+        if ( !strcmp(argv[i],"-f") ) {
+          if ( ++i < argc && isascii(argv[i][0]) && argv[i][0]!='-' ) {
+            if ( !MMGS_Set_inputParamName(mesh,argv[i]) )
+              return 0;
+          }
+          else {
+            fprintf(stderr,"\nMissing filename for %s\n",argv[i-1]);
+            MMGS_usage(argv[0]);
+            return 0;
+          }
+        }
+        break;
       case 'h':
-        if ( !strcmp(argv[i],"-hmin") && ++i < argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hmin,
-                                    atof(argv[i])) )
-            return 0;
-        }
-        else if ( !strcmp(argv[i],"-hmax") && ++i < argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hmax,
-                                    atof(argv[i])) )
-            return 0;
-        }
-        else if ( !strcmp(argv[i],"-hsiz") && ++i < argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hsiz,
-                                    atof(argv[i])) )
-            return 0;
-
-        }
-        else if ( !strcmp(argv[i],"-hausd") && ++i <= argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hausd,
-                                    atof(argv[i])) )
-            return 0;
-        }
-        else if ( !strcmp(argv[i],"-hgradreq") && ++i <= argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hgradreq,
-                                    atof(argv[i])) )
-            return 0;
-        }
-        else if ( !strcmp(argv[i],"-hgrad") && ++i <= argc ) {
-          if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_hgrad,
-                                    atof(argv[i])) )
-            return 0;
-        }
-        else {
-          MMGS_usage(argv[0]);
+        param = MMG5_UNSET;
+        if ( i >= argc -1 ) {
+          fprintf(stderr,"\nMissing argument option %s\n",argv[i]);
           return 0;
         }
+        else {
+          if ( !strcmp(argv[i],"-hmin") ) {
+            param = MMGS_DPARAM_hmin;
+          }
+          else if ( !strcmp(argv[i],"-hmax") ) {
+            param = MMGS_DPARAM_hmax;
+          }
+          else if ( !strcmp(argv[i],"-hsiz") ) {
+            param = MMGS_DPARAM_hsiz;
+          }
+          else if ( !strcmp(argv[i],"-hausd") ) {
+            param = MMGS_DPARAM_hausd;
+          }
+          else if ( !strcmp(argv[i],"-hgradreq") ) {
+            param = MMGS_DPARAM_hgradreq;
+          }
+          else if ( !strcmp(argv[i],"-hgrad") ) {
+            param = MMGS_DPARAM_hgrad;
+          }
+          else {
+            /* Arg unknown by Mmg: arg starts with -h but is not known */
+            MMGS_usage(argv[0]);
+            return 0;
+          }
+
+          assert ( param != MMG5_UNSET );
+
+          val = strtof(argv[i+1],&endptr);
+          if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+            ++i;
+            if ( !MMGS_Set_dparameter(mesh,met,param,val) ){
+              return 0;
+            }
+          } else {
+            /* argument is not a number */
+            fprintf(stderr,"\nMissing argument option %s\n",argv[i]);
+            return 0;
+          }
+        }
+
         break;
       case 'd':
         if ( !strcmp(argv[i],"-default") ) {
@@ -211,7 +249,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
             if ( !MMGS_Set_iparameter(mesh,met,MMGS_IPARAM_verbose,5) )
               return 0;
           }else{
-            fprintf(stderr,"Missing filname for %c%c\n",argv[i-1][1],argv[i-1][2]);
+            fprintf(stderr,"\nMissing filname for %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
@@ -236,29 +274,35 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
         if ( !strcmp(argv[i],"-ls") ) {
           if ( !MMGS_Set_iparameter(mesh,met,MMGS_IPARAM_iso,1) )
             return 0;
-          if ( ++i < argc && (isdigit(argv[i][0]) ||
-                              (argv[i][0]=='-' && isdigit(argv[i][1])) ) ) {
-            if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_ls,atof(argv[i])) )
-              return 0;
+
+          if ( i < argc -1 ) {
+            val = strtof(argv[i+1],&endptr);
+            if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+              ++i;
+              if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_ls,val))
+                return 0;
+            }
           }
-          else i--;
         }
         else if ( !strcmp(argv[i],"-lssurf") ) {
           if ( !MMGS_Set_iparameter(mesh,met,MMGS_IPARAM_isosurf,1) )
             return 0;
-          if ( ++i < argc && (isdigit(argv[i][0]) ||
-                              (argv[i][0]=='-' && isdigit(argv[i][1])) ) ) {
-            if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_ls,atof(argv[i])) )
-              return 0;
+
+          if ( i < argc -1 ) {
+            val = strtof(argv[i+1],&endptr);
+            if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+              ++i;
+              if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_ls,val))
+                return 0;
+            }
           }
-          else i--;
         }
         break;
       case 'm':
         if ( !strcmp(argv[i],"-met") ) {
           if ( !met ) {
-            fprintf(stderr,"No metric structure allocated for %c%c%c option\n",
-                    argv[i-1][1],argv[i-1][2],argv[i-1][3]);
+            fprintf(stderr,"\nNo metric structure allocated for %s option\n",
+                    argv[i-1]);
             return 0;
           }
           if ( ++i < argc && isascii(argv[i][0]) && argv[i][0]!='-' ) {
@@ -266,7 +310,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
               return 0;
           }
           else {
-            fprintf(stderr,"Missing filname for %c%c%c\n",argv[i-1][1],argv[i-1][2],argv[i-1][3]);
+            fprintf(stderr,"\nMissing filname for %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
@@ -277,7 +321,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
             return 0;
         }
         else {
-          fprintf(stderr,"Missing argument option %c\n",argv[i-1][1]);
+          fprintf(stderr,"\nMissing argument option %s\n",argv[i-1]);
           MMGS_usage(argv[0]);
           return 0;
         }
@@ -294,7 +338,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
               return 0;
           }
           else {
-            fprintf(stderr,"Missing argument option %c\n",argv[i-1][1]);
+            fprintf(stderr,"\nMissing argument option %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
@@ -327,8 +371,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
             if ( !MMGS_Set_outputMeshName(mesh,argv[i]) )
               return 0;
           }else{
-            fprintf(stderr,"Missing filname for %c%c%c\n",
-                    argv[i-1][1],argv[i-1][2],argv[i-1][3]);
+            fprintf(stderr,"\nMissing filname for %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
@@ -338,7 +381,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
             return 0;
         }
         else {
-          fprintf(stderr,"Unrecognized option %s\n",argv[i]);
+          fprintf(stderr,"\nUnrecognized option %s\n",argv[i]);
           MMGS_usage(argv[0]);
           return 0;
         }
@@ -347,11 +390,14 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
         if ( !strcmp(argv[i],"-rmc") ) {
           if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_rmc,0) )
             return 0;
-          if ( ++i < argc && (isdigit(argv[i][0]) ) ) {
-            if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_rmc,atof(argv[i])) )
-              return 0;
+          if ( i < argc -1 ) {
+            val = strtof(argv[i+1],&endptr);
+            if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+              ++i;
+              if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_rmc,val))
+                return 0;
+            }
           }
-          else i--;
         }
 #ifdef USE_SCOTCH
         else if ( !strcmp(argv[i],"-rn") ) {
@@ -361,20 +407,20 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
                 return 0;
             }
             else {
-              fprintf(stderr,"Missing argument option %s\n",argv[i-1]);
+              fprintf(stderr,"\nMissing argument option %s\n",argv[i-1]);
               MMGS_usage(argv[0]);
               return 0;
             }
           }
           else {
-            fprintf(stderr,"Missing argument option %s\n",argv[i-1]);
+            fprintf(stderr,"\nMissing argument option %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
         }
 #endif
         else {
-          fprintf(stderr,"Unrecognized option %s\n",argv[i]);
+          fprintf(stderr,"\nUnrecognized option %s\n",argv[i]);
           MMGS_usage(argv[0]);
           return 0;
         }
@@ -389,7 +435,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
               return 0;
           }
           else {
-            fprintf(stderr,"Missing filname for %c%c%c\n",argv[i-1][1],argv[i-1][2],argv[i-1][3]);
+            fprintf(stderr,"\nMissing filname for %s\n",argv[i-1]);
             MMGS_usage(argv[0]);
             return 0;
           }
@@ -405,7 +451,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
             i--;
         }
         else {
-          fprintf(stderr,"Missing argument option %c\n",argv[i-1][1]);
+          fprintf(stderr,"\nMissing argument option %s\n",argv[i-1]);
           MMGS_usage(argv[0]);
           return 0;
         }
@@ -414,10 +460,18 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
         if ( !strcmp(argv[i],"-xreg") ) {
           if ( !MMGS_Set_iparameter(mesh,met,MMGS_IPARAM_xreg,1) )
             return 0;
+          if ( i < argc -1 ) {
+            val = strtof(argv[i+1],&endptr);
+            if ( endptr == &(argv[i+1][strlen(argv[i+1])]) ) {
+              ++i;
+              if ( !MMGS_Set_dparameter(mesh,met,MMGS_DPARAM_xreg,val))
+                return 0;
+            }
+          }
         }
         break;
       default:
-        fprintf(stderr,"Unrecognized option %s\n",argv[i]);
+        fprintf(stderr,"\nUnrecognized option %s\n",argv[i]);
         MMGS_usage(argv[0]);
         return 0;
       }
@@ -436,7 +490,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
           return 0;
       }
       else {
-        fprintf(stdout,"Argument %s ignored\n",argv[i]);
+        fprintf(stdout,"\nArgument %s ignored\n",argv[i]);
         MMGS_usage(argv[0]);
         return 0;
       }
@@ -454,7 +508,7 @@ int MMGS_parsar(int argc,char *argv[],MMG5_pMesh mesh,MMG5_pSol met,MMG5_pSol so
   }
 
   if ( mesh->namein == NULL ) {
-    fprintf(stdout,"  -- INPUT MESH NAME ?\n");
+    fprintf(stdout,"\n  -- INPUT MESH NAME ?\n");
     fflush(stdin);
     MMG_FSCANF(stdin,"%127s",namein);
     if ( !MMGS_Set_inputMeshName(mesh,namein) )
@@ -742,8 +796,8 @@ int MMGS_Get_adjaVerticesFast(MMG5_pMesh mesh, MMG5_int ip,MMG5_int start, MMG5_
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
- * \param met pointer toward the solution structure.
+ * \param mesh pointer to the mesh structure.
+ * \param met pointer to the solution structure.
  * \param ani 1 for aniso metric, 0 for iso one
  *
  * \return 0 if fail, 1 if succeed.
@@ -791,8 +845,8 @@ int MMGS_solTruncatureForOptim(MMG5_pMesh mesh, MMG5_pSol met,int ani) {
 
 
 /**
- * \param mesh pointer toward the mesh
- * \param met pointer toward the metric
+ * \param mesh pointer to the mesh
+ * \param met pointer to the metric
  *
  * \return 1 if succeed, 0 if fail
  *
@@ -866,7 +920,7 @@ int MMGS_doSol_iso(MMG5_pMesh mesh,MMG5_pSol met) {
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param k index of starting triangle
  * \param i local index of point \a p1 in \a k
  * \param p1 point on which we want to compute the 3D unit tensor
@@ -977,7 +1031,7 @@ int MMGS_unitTensor_3D( MMG5_pMesh mesh,MMG5_int k,int i,MMG5_pPoint p1,double *
 }
 
 /**
- * \param mesh pointer toward the mesh structure.
+ * \param mesh pointer to the mesh structure.
  * \param p0 starting point
  * \param k index of starting element
  * \param i local index of \a p0 in \a k
@@ -1077,7 +1131,7 @@ int MMGS_surfopenballRotation(MMG5_pMesh mesh,MMG5_pPoint p0,MMG5_int k, int i,
 }
 
 /**
- * \param mesh pointer toward the mesh
+ * \param mesh pointer to the mesh
  * \param k index of starting triangle
  * \param i local index of point \a p1 in \a k
  * \param p1 point on which we want to compute the 3D unit tensor
@@ -1265,8 +1319,8 @@ int MMGS_unitTensor_2D ( MMG5_pMesh mesh,MMG5_int k,int i,MMG5_pPoint p1,
 
 
 /**
- * \param mesh pointer toward the mesh
- * \param met pointer toward the metric
+ * \param mesh pointer to the mesh
+ * \param met pointer to the metric
  *
  * \return 1 if succeed, 0 if fail
  *
